/****** TransCom *************************************************************
 * CDPRO II DSA control module
 *
 * DSA PINOUT:
 *  1 - B4 - ACK
 *  2 - B0 - DDA - DATA
 *  3 - B3 - DST - STB
 *  4 - NC
 *  5 - GND
 *  6 - B5 - LASER OFF!
 *
 ****************************************************************************
 */

#include <pic18.h>

/*******************************
 * PIC Configuration Settings
 *******************************
 */
__CONFIG(1,HS);                 // HS oscillator
__CONFIG(2,PWRTEN & BOREN & BORV42 & WDTDIS);  // Resets & disable watchdog
__CONFIG(3,CCP2RC1);            // CCP2 to RC1 (rather than to RB3)
__CONFIG(4,LVPDIS & DEBUGEN);   // RB5 enabled for I/O

/*******************************
 * Definitions and equates
 *******************************
 */

#define DUMMY 0x00
#define PLUS  11				// Value used to indicate +10 key
#define PLAY  12                // Value used to indicate PLAY key
#define STOP  13                // Value used to indicate STOP key
#define PAUSE 14				// Value used to indicate PAUSE key
#define FWD   15				// Value used to indicate FWD key
#define BACK  16				// Value used to indicate BACK key


/*******************************
 * Global variables
 *******************************
 */

char ALIVECNT;                  // Counter for blinking "Alive" LED
char COUNTER;					// Counter used for debugging button routine
char PRESSED;					// Flag for keeping track of button presses
char SCOMMAND;					// Command to be sent
char DATAOUT;					// Data to be sent
char RCOMMAND;					// Commed recieved from transport
char DATAIN;					// Data recieved from transport
char DATAREC;					// Flag set if transport recieved data
char CURTRACK;					// Current track number
char MAXTRACK;					// Maximum track number
char MINTRACK;					// Minimum track number
char CDMIN;						// CD minutes
char CDSEC;						// CD seconds
char CDFRM;						// CD frames
char FIRSTPLAY;					// Flag to tell if it is the first time pressing play
char PAUSEFLAG;					// Flag to tell if transport is paused
char CONV2NUM;
char POSITION;
char PLAYING;
char PAUSETRACK;
char INTRCOMMAND;
char INTCONV2NUM;
char INTPOSITION;
char INTDATAIN;
char PLAYINUM;
char WAIT;
char LONG;
char NEXTFLAG;
char PREVFLAG;

char INTSCOMMAND;					// Command to be sent
char INTDATAOUT;					// Data to be sent

char PLUSTEN;
char IRLONG;                      // Counter for repeated interrupt loops
char REMSTATE;                  // Variable to track interrupts
char RC_CODE;                   // Variable to store code from remote
char KEYCODE;                   // Converted key value
int DECODED_KEYPRESS;					  // Signifies a decoded and displayed keypress.

/*******************************
 * Variable arrays
 *******************************
 */
char NUMHstr[] = { 0x88,0,0,0,0,0,0,0,0,0 };  // Array ADRESH Value
char NUMLstr[] = { 0xc8,0,0,0,0,0,0,0,0,0 };  // Array ADRESL Value
char LONGstr[] = { 0x8b,0,0,0,0,0,0,0,0,0 };
char LONGstr2[] = { 0xcb,0,0,0,0,0,0,0,0,0 };
char DISNUM[] = { 0xc6,0,0,0 };
char INTDISNUM[] = { 0xc6,0,0,0 };
char PLAYICON[] = { 0x89,0,0,0 };

/*******************************
 * Constant strings
 *
 * For stability reasons, create an EVEN number of elements in any given array
 *******************************
 */


const char LCDstr[]  = {0x33,0x32,0x28,0x01,0x0c,0x06,0x00,0x00};			// LCD Initialization string
const char PLAYstr[] = {0x80,'P','L','A','Y',' ',' ',' ',' ',' ',' ',0};	// "PLAYING" on first line
const char STOPstr[] = {0x80,'S','T','O','P',' ',' ',' ',' ',' ',' ',0};	// "STOPPED" on first line
const char PAUSEstr[] = {0x80,'P','A','U','S','E',' ',' ',' ',' ',' ',0};  	// "PAUSED" on first line
const char CDENDstr[] = {0x80,'E','N','D',' ','O','F',' ','C','D',' ',0}; 	// "END OF CD" on first line
const char LOADstr[] = {0x80,'L','O','A','D','I','N','G',' ','C','D',0};	// "LOADING CD" on first line
const char READYstr[] = {0x80,'C','D',' ','R','E','A','D','Y',' ',' ',0};	// "CD READY" on first line
const char ERRstr[] = {0x80,'E','R','R','O','R','!','!',' ',' ',' ',0};		// "ERROR!!" on first line
const char TRACKstr[] = {0xc0,'T','r','a','c','k',' ','0','0',0};			// "Track 00" on second line
const char TIMEstr[] = {0xca,'T','i','m','e',' ','0','0',':','0','0',0};	// "Time 00:00" on second line
const char SENDerr[] = {0x80,'S','e','n','d',' ','E','r','r','o','r',0}; 	// "Send Error" on first line
const char Test[] = {0x82,'T','e','s','t',' ',' ',' ',' ',' ',' ',' ',' ',0};
const char ClrDisVar[] = {0x8b,' ',' ',' ',' ',' ',' ',' ',' ',0};
const char Plus10str[] = {0x8e,'+','1','0',' ',0,};

const char str1[] = {0x48,0x9f,0x8e,0x84,0x80,0x84,0x8e,0x9f,0x80,0x00};
const char str2[] = {0x50,0x87,0x86,0x84,0x80,0x84,0x8c,0x9c,0x08,0x00};
const char str3[] = {0x58,0x80,0x81,0x83,0x9b,0x98,0x90,0x80,0x80,0x00};
const char str4[] = {0x60,0x80,0x91,0x9b,0x9b,0x9b,0x91,0x80,0x80,0x00};
const char str5[] = {0x68,0x80,0x90,0x98,0x9b,0x83,0x81,0x80,0x80,0x00};
const char str6[] = {0x70,0x9c,0x8c,0x84,0x80,0x84,0x86,0x87,0x80,0x00};

const char KeyTable[] = {0b00000000,1,
                         0b00000001,2,
                         0b00000010,3,
                         0b00000011,4,
                         0b00000100,5,
                         0b00000101,6,
                         0b00000110,7,
                         0b00000111,8,
                         0b00001000,9,
                         0b00001001,10,
                         0b00001011,PLUS,
                         0b00011010,PLAY,
                         0b00011000,STOP,
                         0b00011100,FWD,
                         0b00011011,BACK,
                         0b00011001,PAUSE,
                         0xff,0xff};

/*******************************
 * Function prototypes
 *******************************
 */

void Initial(void);
void InitLCD(void);
void DisplayC(const char *);
void DisplayV(char *);
void BlinkAlive(void);
void LoopTime(void);

void IR(void);
char LookUpCode(void);

void DetectButton(void);
void SendData(void);
void GetData(void);
void Play(void);
void Stop(void);
void Prev(void);
void Next(void);
void Pause(void);
void ReadTOC(void);
void Search(void);
void Volume(void);
void DisNum(void);
void GetTime(void);
void PlayIcon(void);
void GetError(void);
void IntGetData(void);
void IntDisplayV(char *);
void IntDisplayC(const char *);
void IntDisNum(void);
void IntT40(void);
void ClearLCD(void);

//Debugging functions
void DisVar(char);
void IntDisVar(char);
void DisVar2(char);

/*******************************
 * main()
 *******************************
 */

void main()
{
    Initial();              // Initialize everything

    while(1)
    {
        GIEH=0;
        BlinkAlive();		// Blink Alive LED
        //PlayIcon();
        DetectButton();		// Dectect if a button has been pressed

        if (IRLONG > 0)       // If LONG > 0,
        {
            IRLONG--;         //  then decrement
        }
        IR();

        TRISB = TRISB | 0b00000001;

        GIEH=1;
        LoopTime();         // Delay for 10 ms
    }
}

/*******************************
 * Initial()
 *
 * This subroutine performs all initializations of variables and registers.
 *******************************
 */

void Initial()
{
    ADCON0 = 0b01000001;        // Select RA0(AN0) for analog input
	ADCON1 = 0b11001110;        // Enable PORTA & PORTE as digital I/O pins (pin RA0 is analog)
    TRISA = 0b11101001;         // Set I/O for PORTA
    TRISB = 0b11010111;         // Set I/O for PORTB
    TRISC = 0b11010010;         // Set I/O for PORTC
    TRISD = 0b00001111;         // Set I/O for PORTD
    TRISE = 0b00000100;         // Set I/O for PORTE
    T1CON = 0b00000000;         // Set up CCP1/Timer1 for frequency generation
    CCP1CON = 0b00001000;       // Select compare mode for CCP1/Timer1
    T3CON = 0b10001001;         // Set up CCP2/Timer3 for IR signal capture
    CCP2CON = 0b00000100;       // Enable capture mode for falling edge of IR
    PR2 = 222;                  // Set B=223 for Timer2 scalar
    T2CON = 0b00110111;         // Set A=16, C=7 for Timer 2 scalar
    EECON1 = 0;                 // Set up EECON1 for EEPROM access
    EEIF = 1;                   // Clear write-operation flag
    IPEN = 1;                   // Enable priority levels for interrupts
    TMR1IP = 0;                 // Assign low priority to TMR1 interrupts
    CCP1IP = 0;                 //  and to CCP1 interrupts
    CCP2IP = 1;                 // Assign high priority to CCP2 interrupts
    ALIVECNT = 250;             // Setup counter for blinking "Alive" LED

    PRESSED = 0;				// Clear pressed flag
	DATAREC = 0;				// Clear data received flag
	CURTRACK = 0x01;			// Set current track to 1
	FIRSTPLAY = 0;				// Clear first play flag
	RB0 = 1;					// Set data line high
	RB3 = 1;					// Set strobe line high
	RB5 = 0;					// Make sure laser is off
	PAUSEFLAG = 0;				// Clear pause flag
	PLAYING = 0;				// Clear playing flag
	PLAYINUM = 1;
	WAIT = 0;
	LONG = 1;
	NEXTFLAG = 0;
	PREVFLAG = 0;

	//Interupt setup
	INT0IE = 1;					// Enable external interupts on RB0/INT0
	INT0IF = 0;					// Clear INT0 interrupt flag
	INTEDG0 = 0;				// Set interrupt on falling edge

	CCP2IE = 1;					// Enable CCP2 interrupts (IR)

    InitLCD();                  // Initialize LCD display
	DisplayC(str1);
	DisplayC(str2);
	DisplayC(str3);
	DisplayC(str4);
	DisplayC(str5);
	DisplayC(str6);
    DisplayC(TRACKstr);			// Display "Track " on LCD
    DisplayC(TIMEstr);			// Display "Time   :  " on LCD

    IRLONG = 10;					// initializing for IR subroutine
    REMSTATE = 0;
    PLUSTEN = 0;
}//end Initial()

/*******************************
 * InitLCD()
 *
 * Initialize the Optrex 20x2 character LCD.
 * First wait for 0.1 second, to get past display's power-on reset time.
 *******************************
 */

void InitLCD()
{
    char currentChar;
    const char *tempPtr;
    char i;

    i = 10;                      // Wait 0.1 second
    while (i)
    {
        LoopTime();              // Call LoopTime() 10 times
        i--;
    }

    RE0 = 0;                     // RS=0 for command
    tempPtr = LCDstr;

    while (*tempPtr)             // if the byte is not zero
    {
        currentChar = *tempPtr;
        RE1 = 1;                 // Drive E pin high
        PORTD = currentChar;     // Send upper nibble
        RE1 = 0;                 // Drive E pin low so LCD will accept nibble
        LoopTime();
        currentChar <<= 4;       // Shift lower nibble to upper nibble
        RE1 = 1;                 // Drive E pin high again
        PORTD = currentChar;     // Write lower nibble
        RE1 = 0;                 // Drive E pin low so LCD will process byte
        LoopTime();              // Wait 40 usec
        tempPtr++;               // Increment pointerto next character
    }
}//end InitLCD()

/*******************************
 * T40()
 *
 * Pause for 40 microseconds  or 40/0.4 = 100 clock cycles.
 * Assumes 10/4 = 2.5 MHz internal clock rate.
 *******************************
 */

void T40()
{
    unsigned char cCOUNT = 7;
    while (cCOUNT)
    cCOUNT--;
}//end T40()

/*******************************
 * DisplayC(const char *)
 *
 * This subroutine is called with the passing in of an array of a constant
 * display string.  It sends the bytes of the string to the LCD.  The first
 * byte sets the cursor position.  The remaining bytes are displayed, beginning
 * at that position.
 * This subroutine expects a normal one-byte cursor-positioning code, 0xhh, or
 * an occasionally used two-byte cursor-positioning code of the form 0x00hh.
 *******************************
 */

void DisplayC(const char * tempPtr)
{
    char currentChar;

    RE0 = 0;                   // Drive RS pin low for cursor-pos code
    while (*tempPtr)           // if the byte is not zero
    {
        currentChar = *tempPtr;
        RE1 = 1;               // Drive E pin high
        PORTD = currentChar;   // Send upper nibble
        RE1 = 0;               // Drive E pin low so LCD will accept nibble
        currentChar <<= 4;     // Shift lower nibble to upper nibble
        RE1 = 1;               // Drive E pin high again
        PORTD = currentChar;   // Write lower nibble
        RE1 = 0;               // Drive E pin low so LCD will process byte
        T40();                 // Wait 40 usec
        RE0 = 1;               // Drive RS pin high for displayable chars
        tempPtr++;             // Increment pointerto next character
    }
}//end DisplayC()

/*******************************
 * DisplayV(char *)
 *
 * This subroutine is called with the passing in of an array of a variable
 * display string.  It sends the bytes of the string to the LCD.  The first
 * byte sets the cursor position.  The remaining bytes are displayed, beginning
 * at that position.
 *******************************
 */

void DisplayV(char * tempPtr)
{
    char currentChar;

    RE0 = 0;                   // Drive RS pin low for cursor-pos. code
    while (*tempPtr)           // if the byte is not zero
    {
        currentChar = *tempPtr;
        RE1 = 1;               // Drive E pin high
        PORTD = currentChar;   // Send upper nibble
        RE1 = 0;               // Drive E pin low so LCD will accept nibble
        currentChar <<= 4;     // Shift lower nibble to upper nibble
        RE1 = 1;               // Drive E pin high again
        PORTD = currentChar;   // Write lower nibble
        RE1 = 0;               // Drive E pin low so LCD will process byte
        T40();                 // Wait 40 usec
        RE0 = 1;               // Drive RS pin high for displayable chars
        tempPtr++;             // Increment pointerto next character
    }
}//end DisplayV()

/*******************************
 * BlinkAlive()
 *
 * This subroutine briefly blinks the LED next to the PIC every two-and-a-half
 * seconds.
 *******************************
 */

void BlinkAlive()
{
    RA4 = 1;              // Turn off LED
    if (!(--ALIVECNT))    // Decrement loop counter, return if not 0
    {
        ALIVECNT = 250;   // Reinitialize ALIVECNT
        RA4 = 0;          // Turn on LED for ten ms every 2.5 sec
    }
}//end BlinkAlive()


/********************************
 * LoopTime()
 *
 * This subroutine waits for the TMR2IF flag to indicate that 10ms have passed
 * since the last time this occurred.
 ********************************
 */

void LoopTime()
{
    while (!TMR2IF) { }     // Wait until TMR2IF flag is set
    TMR2IF = 0;             // Clear flag
}//end LoopTime()


/********************************
 * IR() - IR subroutine
 *
 ********************************
 */
void IR()
{
    char CodeVal;

    if (IRLONG != 0 || REMSTATE != 1) // Return if no new key is ready
    {
        return;
    }
    REMSTATE = 0;                   // Reset REMSTATE to only use this
                                    //  key press once
    CodeVal = LookUpCode();
    DisVar(RC_CODE);

    if (CodeVal == 0xff)   // Convert code & check for errors
    {
        return;                     // Invalid key detected, so return
    }

    DisplayC(ClrDisVar);

    if (CodeVal == PLAY)             // Handle Play key
    {
		if(FIRSTPLAY == 0){				// If this is first press of play
			Play();						// Run play routine
		}
		Play();
    }
    else if (CodeVal == PLUS)
   	{
	   	PLUSTEN = !PLUSTEN;
   	}
    else
    {
	    if (CodeVal == STOP)             // Handle Stop key
	    {
			DisplayC(STOPstr);				// Display stop string
			PRESSED = 1;					// Set pressed flag
			if(FIRSTPLAY == 1){				// If play has been pressed once
				Stop();						// Run stop routine
			}
	    }
	    else if (CodeVal == FWD)             // Handle Prev key
	    {
			PRESSED = 1;					// Set pressed flag
			NEXTFLAG = 1;
	    }
	    else if (CodeVal == BACK)             // Handle Back key
	    {
			PRESSED = 1;					// Set pressed flag
			PREVFLAG = 1;
	    }
	    else if (CodeVal == PAUSE)             // Handle Pause key
	    {
			PRESSED = 1;						// Set pressed flag
			if(FIRSTPLAY == 0){					// If play has not been pressed first time
				Play();							// Run play routine
			}
			Pause();
	    }
	   	else if (CodeVal <= 10)
	   	{
			CURTRACK = CodeVal;

			if (PLUSTEN) CURTRACK = CURTRACK+10;
			if (CURTRACK >= MAXTRACK) CURTRACK = MAXTRACK;

			if(FIRSTPLAY == 0){				// If this is first press of play
				Play();						// Run play routine
			}
			Play();
	   	}
	   	PLUSTEN = 0;
   	}

   	if (PLUSTEN) DisplayC(Plus10str);
   	else DisplayC(ClrDisVar);
}

/*********************************
 * LookUpCode()
 *
 * This subroutine uses the lookup table to retrieve the meaning of the code
 * stored in RC_CODE.  It then returns this value to the calling function.
 *********************************
 */
char LookUpCode()
{
    char i = 0;                     // Index for table

    do                              // Loop through table until code is
    {                               //  found, or until end is reached
        if(KeyTable[i] == RC_CODE)  // If code is found,
        {
            return KeyTable[i+1];   //  then return "meaning"
        }
        else                        // Otherwise,
        {
            i += 2;                 //  increment index
        }
    } while (KeyTable[i] != 0xff);  // 0xff indicates end of table

    return 0xff;                    // Return 0xff indicating an invalid key
}

/********************************
* DetectButton()
*
* This subroutine checks to see if a button has been pressed on the A/D input
* It then determines which button has been pressed.
* Also keeps track if it is the same press or a new press.
*********************************/

void DetectButton()
{
	ADCON0 = 0b01000101;						// Tell A/D to take a measurement
	while (ADCON0 != 0b01000001){}				// Wait for measurement to finish

	if(PRESSED == 0){							// If this is a new press
		if(ADRESH == 0){						// If the high byte for the measurement = 0
			if(ADRESL >= 0 && ADRESL <= 80){	// If the low byte is between 11 and 100
				// insert open/close code here
			}
			if(ADRESL > 80 && ADRESL <= 144){	// If the low byte is between 11 and 100
				DisplayC(STOPstr);				// Display stop string
				PRESSED = 1;					// Set pressed flag
				if(FIRSTPLAY == 1){				// If play has been pressed once
					Stop();						// Run stop routine
				}
			}
			if(ADRESL > 144 && ADRESL <= 208){	// If the low byte is between 11 and 100
				PRESSED = 1;
				PREVFLAG=1;
			}
			if(ADRESL > 208){					// If the low byte is > 208
				PRESSED = 1;						// Set pressed flag
				if(FIRSTPLAY == 0){					// If play has not been pressed first time
					Play();							// Run play routine
				}
				Play();							// Run pause routine
			}
		}
		else if(ADRESH == 1) {
			if(ADRESL >= 0 && ADRESL <= 10){	// If the low byte is between 0 and 10
				PRESSED = 1;					// Set pressed flag
				if(FIRSTPLAY == 0){				// If this is first press of play
					Play();						// Run play routine
				}
				Play();							// Run play routine
			}
			if(ADRESL > 10 && ADRESL <= 80){	// If the low byte is between 11 and 100
				PRESSED = 1;
				NEXTFLAG=1;
			}
			if(ADRESL > 80 && ADRESL <= 144){					// If the low byte is > 208
				PRESSED = 1;						// Set pressed flag
				if(FIRSTPLAY == 0){					// If play has not been pressed first time
					Play();							// Run play routine
				}
				Pause();							// Run pause routine
			}
			if(ADRESL > 144 && ADRESL <= 208){	// If the low byte is between 101 and 150
				PRESSED = 1;					// Set pressed flag
				if(FIRSTPLAY == 0){				// If play has not been pressed first time
					Play();						// Run play routine
				}
				Prev();							// Run previous routine
				if(PLAYING == 1){
					Play();
				}
			}
			if(ADRESL > 208){	// If the low byte is between 151 and 200
				PRESSED = 1;					// Set pressed flag
				if(FIRSTPLAY == 0){				// If play has not been pressed first time
					Play();						// Run play routine
				}
				Next();							// Run next routine
				if(PLAYING == 1){
					Play();
				}
			}
		}
	}
	else {
		if(ADRESH < 3){
			Search();
		}
		else {									// If high byte is greater than 3
			PRESSED = 0;						// Clear pressed flag (No buttons are being pressed)
			if(NEXTFLAG == 1 || PREVFLAG == 1){
				NEXTFLAG = 0;
				PREVFLAG = 0;
				Search();
			}
		}
	}
}//end DetectButton()


/********************************
* Play()
*
* Send command to play current track
*********************************/
void Play()
{
	if(FIRSTPLAY == 1){							// If play has been pressed once
		if (PAUSEFLAG == 0){
			DisplayC(PLAYstr);						// Display play string
			SCOMMAND = 0x01;						// load play command 0x01 to be sent
			DATAOUT = CURTRACK;						// load track number in to data to be sent
			SendData();								// Send the command and data
			GetData();								// Get reponse from transport
			PLAYING = 1;
			CONV2NUM = CURTRACK;					// Set CONV2NUM to CURTRACK
			DisNum();							// Set playing flag
		}
		else {
			Pause();
		}
	}
	else if(FIRSTPLAY == 0){					// If this if first time play has been pressed
		DisplayC(LOADstr);						// display LOADING CD
		RB5 = 1;								// Turn on laser
		ReadTOC();								// Read the table of contents from CD
		FIRSTPLAY = 1;							// Set first play flag
		POSITION = 0xc6;						// Set position code to 0xc6
		CONV2NUM = CURTRACK;					// Set CONV2NUM to CURTRACK
		DisNum();								// Display CURTRACK number
		DisplayC(READYstr);						// display CD READY
	}
}//end Play()


/********************************
* Stop()
*
* Send command to stop playing and clear TOC
* Need to modify clear TOC code later
*********************************/
void Stop()
{
	PLAYING = 0;							// clear playing flag

	//Send Stop command
	SCOMMAND = 0x02;						// load stop command 0x02
	DATAOUT = DUMMY;						// load dummy data
	SendData();								// send command and data to transport
	GetData();								// Get response back from transport

	//Send Clear TOC command
	SCOMMAND = 0x6A;						// load Clear TOC command 0x6A
	DATAOUT = DUMMY;						// load dummy variable
	SendData();								// Send command and data to transport
	GetData();								// Wait for response from transport
	RB5 = 0;								// Turn off laser

	CURTRACK = 0x01;						// Set current track to 1
	FIRSTPLAY = 0;
	ClearLCD();
}//end Stop()

/********************************
* Prev()
*
* Send command to go to previous track
*********************************/
void Prev(){
	if(CURTRACK == 1){						// if the current track equals 1
		CURTRACK = MAXTRACK;				// current track gets max track number
	}
	else{									// if the current track is greater than 1
		CURTRACK = CURTRACK - 1;			// current track gets current track minus 1
	}
	ClearLCD();
	POSITION = 0xc6;						// Set position code to 0xc6
	CONV2NUM = CURTRACK;					// Set CONV2NUM to CURTRACK
	DisNum();								// Display converter and display CURTRACK number

}//end Prev()

/********************************
* Next()
*
* Send command to go to next track
*********************************/
void Next(){
	if(CURTRACK == MAXTRACK){				// If the current track equals the max track number
		CURTRACK = MINTRACK;				// current track gets min track number
	}
	else{									// if the current track is less than max track number
		CURTRACK = CURTRACK + 1;			// current track gets current track plus 1
	}
	ClearLCD();
	POSITION = 0xc6;						// Set position code to 0xc6
	CONV2NUM = CURTRACK;					// Set CONV2NUM to CURTRACK
	DisNum();								// Display converter and display CURTRACK number
}//end Next()

/********************************
* Pause()
*
* Send command to pause
*********************************/
void Pause(){
	//Pause
	if(PAUSEFLAG == 0){						// if pause flag is cleared
		RA1=1;
		DisplayC(PAUSEstr);					// display pause
		SCOMMAND = 0x04;					// load pause command
		DATAOUT = DUMMY;					// load dummy data
		SendData();							// send command and data
		GetData();							// get response from transport
		PAUSEFLAG = 1;						// set pause flag
		PLAYING = 0;						// clear playing flag
		PAUSETRACK = CURTRACK;				// store current track
	}
	//Release Pause
	else{									// if pause flag is set
		RA1 = 0;
		DisplayC(PLAYstr);					// display play string
		PAUSEFLAG = 0;						// clear pause flag
		PLAYING = 1;						// set play flag
		if(PAUSETRACK != CURTRACK){			// if stored track is not equal to current track
			Play();							// play the new track
		}
		SCOMMAND = 0x05;					// load release pause command
		DATAOUT = DUMMY;					// load dummy data
		SendData();							// send command and data to transport
		GetData();							// get response from transport
	}
}//end Pause

/********************************
* ReadTOC()
*
* Send command to read the table of contents from the CD
*********************************/
void ReadTOC()
{
	SCOMMAND = 0x03;						// load Read TOC command 0x03
	DATAOUT = 0x00;							// load track number
	SendData();								// send command and data to transport
	GetData();								// get response from transport
	MINTRACK = DATAIN;						// MINTRACK gets DATAIN
	GetData();								// get response from transport
	MAXTRACK = DATAIN;						// MAXTRACK gets DATAIN
	GetData();								// get response from transport
	CDMIN = DATAIN;							// CDMIN (cd minutes) gets DATAIN
	GetData();								// get response from transport
	CDSEC = DATAIN;							// CDSEC (cd seconds) gets DATAIN
	GetData();								// get resoponse from transport
	CDFRM = DATAIN;							// CDFRM (cd frames) gets DATAIN
}//end ReadTOC


/********************************
* Search()
*
* Send command to search the CD
*********************************/
void Search(){
	if(PLAYING == 1){
	if(NEXTFLAG == 1){
		SCOMMAND = 0x06;
		DATAOUT = 1;
		SendData();
	}
	else if(PREVFLAG == 1){
		SCOMMAND = 0x07;
		DATAOUT = 1;
		SendData();
	}
	else{
		SCOMMAND = 0x08;
		DATAOUT = DUMMY;
		SendData();
	}
	}
}//end Search()


/********************************
* ClearLCD()
*
* Clears the LCD
*********************************/
void ClearLCD(){
	POSITION = 0xC6;						// load position code
	CONV2NUM = 0x00;						// load 0 into CONV2NUM
	DisNum();								// display tracks on LCD
	CONV2NUM = 0x00;						// load 0 into CONV2NUM
	POSITION = 0xd2;						// load LCD position
	DisNum();								// display seconds on LCD
	CONV2NUM = 0x00;						// load 0 into CONV2NUM
	POSITION = 0xcf;						// load LCD position
	DisNum();								// display minutes on LCD
}

/********************************
* GetError()
*
* If error is found this routine attempts to find
* what it was and then clears it.
*********************************/

void GetError(){
	DisplayC(ERRstr);
	SCOMMAND = 0x16;
	DATAOUT = DUMMY;
	SendData();
	GetData();
	if(RCOMMAND == 0x04){
		//DisVar(DATAIN);
	}
	SCOMMAND = 0x17;
	DATAOUT = DUMMY;
	SendData();
	GetData();
}

/********************************
* DisNum()
*
* Takes any number between 0 - 99 converts to ASCII
* and then displays in on the LCD at the position
* specified by POSITION variable
*********************************/
void DisNum(){
	char remainder;							// initalize remainder variable

	remainder = CONV2NUM % 10;				// mod CONV2NUM with 10 store result into remainder
	DISNUM[2] = remainder | 0b00110000;		// OR remainder with 0x30 to convert to ASCII and store in string
	CONV2NUM = CONV2NUM / 10;				// divide CONV2NUM by 10
	remainder = CONV2NUM % 10;				// mode CONV@NUM and store result into remainder
	DISNUM[1] = remainder | 0b00110000;		// OR remainder with 0x03 to convert to ASCII and store in string
	DISNUM[0] = POSITION;					// load position code into string
	DisplayV(DISNUM);						// Display new string
}//end DisNum()


/********************************
* SendData()
*
* This routine sends data to the CD transport
*********************************/
void SendData()
{
	//intialize variables
	char datacount;
	char tempcommand;
	char tempdata;

	TRISB = TRISB | 0b00010000;						// set RB4 to an input
	TRISB = TRISB & 0b11110111;						// set RB3 to output
	TRISB = TRISB & 0b11111110;						// set RB0 to output

	//Sync the PIC with the transport
	RB0 = 0;										// clear DATA line
	while(RB4 == 1){}								// wait for transport to clear ACK line
	RB0 = 1;										// set DATA line
	while(RB4 == 0){}								// wait for transport to set ACK line

	tempcommand = SCOMMAND;							// tempcommand gets command to be sent
	tempdata = DATAOUT;								// tempdata gets data to be sent
	//Send Command
	for(datacount = 0; datacount < 8; datacount++)	// while datacount is less than 8
	{
		//load bit on to DATA line
		if((tempcommand & 0b10000000) == 0){		// if tempcommand AND 0x80 equals 0
			RB0 = 0;								// clear RB0
		}
		else{										// else if it equals 1
			RB0 = 1;								// set RB0
		}
		tempcommand = tempcommand << 1;				// shift tempcommand left once
		RB3 = 0;									// clear STB line
		while(RB4 == 1){}							// wait for ACK line to be cleared
		RB3 = 1;									// set STB line
		while(RB4 == 0){}							// wait for ACK line to be set
	}
	//Send Data
	for(datacount = 0; datacount < 8; datacount++)	// while datacount is less than 8
	{
		//load bit on to DATA line
		if((tempdata & 0b10000000) == 0){			// if tempdata AND 0x80 equals 0
			RB0 = 0;								// clear RB0
		}
		else{										// else if it equals 1
			RB0 = 1;								// set RB0
		}
		tempdata = tempdata << 1;					// shift tempdata left once
		RB3 = 0;									// clear STB line
		while(RB4 == 1){}							// wait for ACK line to be cleared
		RB3 = 1;									// set STB line
		while(RB4 == 0){}							// wait for ACK line to be set
	}

	TRISB = TRISB | 0b00001000;						// set RB3 to input
	TRISB = TRISB & 0b11101111;						// set RB4 to an output
	TRISB = TRISB | 0b00000001;						// set RB0 to an input


	//Get acknowledgement from transport
	RB4 = 0;										// clear ACK line
	while(RB3 == 1){}								// wait for STB to clear

	//If data WAS recieved
	if(RB0 == 1){
		RB4 = 1;									// set the ACK line
		DATAREC = 1;								// set DATAREC
	}
	//If data WAS NOT recieved
	else if(RB0 == 0){
		RB4 = 1;									// set the ACK line
		DATAREC = 0;								// clear DATAREC
	}
}//end SendData


/********************************
* GetData()
*
* This routine gets data from the CD transport
*********************************/
void GetData()
{
	char datacount;

	RCOMMAND = 0b00000000;							// initallize RCOMMAND to 0
	DATAIN = 0b00000000;							// initallize DATAIN to 0

	TRISB = TRISB | 0b00001000;						// set RB3 to input
	TRISB = TRISB & 0b11101111;						// set RB4 to an output
	TRISB = TRISB | 0b00000001;						// set RB0 to an input

	//Sync the PIC with the transport
	while(RB0 == 1){}								// Wait for DATA line to go low
	RB4 = 0;										// Clear the ACK line
	while(RB0 == 0){}								// Wait for DATA line to go high
	RB4 = 1;										// Set the ACK line

	//Recieve Command
	for(datacount = 0; datacount < 8; datacount++)
	{
		while(RB3 == 1){}							// Wait for STB line to be cleared
		RCOMMAND = RCOMMAND << 1;					// Shift RCOMMAND once to the left
		if(RB0 == 1){								// If data bit equals 1
			RCOMMAND = RCOMMAND | 0b00000001;		// RCOMMAND gets RCOMMAND ORed with 0x01
		}
		else if(RB0 == 0){							// if data bit equals 0
			RCOMMAND = RCOMMAND & 0b11111110;		// RCOMMAND gets RCOMMANd ANDed with 0xfe
		}
		RB4 = 0;									// Clear ACK line
		while(RB3 == 0){}							// Wait for STB line to be set
		RB4 = 1;									// Set ACK line
	}
	//Recieve Data
	for(datacount = 0; datacount < 8; datacount++)
	{
		while(RB3 == 1){}							// Wait for STB line to be cleared
		DATAIN = DATAIN << 1;						// Shift RCOMMAND once to the left
		if(RB0 == 1){								// if data bit equals 1
			DATAIN = DATAIN | 0b00000001;			// DATAIN gets DATAIN ORed with 0x01
		}
		else if(RB0 == 0){							// if data bit equals 0
			DATAIN = DATAIN & 0b11111110;			// DATAIN gets DATAIN ANDed with 0xfe
		}
		RB4 = 0;									// Clear ACK line
		while(RB3 == 0){}							// Wait for STB line to be set
		RB4 = 1;									// Set ACK line
	}

	TRISB = TRISB | 0b00010000;						// Set RB4 to an input
	TRISB = TRISB & 0b11110111;						// Set RB3 to output
	TRISB = TRISB & 0b11111110;						// set RB0 to output

	//Get acknowledgement from transport
	while(RB4 == 1){}								// Wait for ACK line to be cleared
	RB0 = 1;										// Set DATA line
	RB3 = 0;										// Clear STB line
	while(RB4 == 0){}								// Wait for ACK line to be set
	RB0 = 1;										// Make sure DATA line is set
	RB3 = 1;										// Set STB line
}//end GetData

/********************************
* IntSendData()
*
* This routine sends data to the CD transport
*********************************/
void IntSendData()
{
	//intialize variables
	char datacount;
	char tempcommand;
	char tempdata;

	TRISB = TRISB | 0b00010000;						// set RB4 to an input
	TRISB = TRISB & 0b11110111;						// set RB3 to output
	TRISB = TRISB & 0b11111110;						// set RB0 to output

	//Sync the PIC with the transport
	RB0 = 0;										// clear DATA line
	while(RB4 == 1){}								// wait for transport to clear ACK line
	RB0 = 1;										// set DATA line
	while(RB4 == 0){}								// wait for transport to set ACK line

	tempcommand = INTSCOMMAND;							// tempcommand gets command to be sent
	tempdata = INTDATAOUT;								// tempdata gets data to be sent
	//Send Command
	for(datacount = 0; datacount < 8; datacount++)	// while datacount is less than 8
	{
		//load bit on to DATA line
		if((tempcommand & 0b10000000) == 0){		// if tempcommand AND 0x80 equals 0
			RB0 = 0;								// clear RB0
		}
		else{										// else if it equals 1
			RB0 = 1;								// set RB0
		}
		tempcommand = tempcommand << 1;				// shift tempcommand left once
		RB3 = 0;									// clear STB line
		while(RB4 == 1){}							// wait for ACK line to be cleared
		RB3 = 1;									// set STB line
		while(RB4 == 0){}							// wait for ACK line to be set
	}
	//Send Data
	for(datacount = 0; datacount < 8; datacount++)	// while datacount is less than 8
	{
		//load bit on to DATA line
		if((tempdata & 0b10000000) == 0){			// if tempdata AND 0x80 equals 0
			RB0 = 0;								// clear RB0
		}
		else{										// else if it equals 1
			RB0 = 1;								// set RB0
		}
		tempdata = tempdata << 1;					// shift tempdata left once
		RB3 = 0;									// clear STB line
		while(RB4 == 1){}							// wait for ACK line to be cleared
		RB3 = 1;									// set STB line
		while(RB4 == 0){}							// wait for ACK line to be set
	}

	TRISB = TRISB | 0b00001000;						// set RB3 to input
	TRISB = TRISB & 0b11101111;						// set RB4 to an output
	TRISB = TRISB | 0b00000001;						// set RB0 to an input


	//Get acknowledgement from transport
	RB4 = 0;										// clear ACK line
	while(RB3 == 1){}								// wait for STB to clear

	//If data WAS recieved
	if(RB0 == 1){
		RB4 = 1;									// set the ACK line
		INTDATAREC = 1;								// set DATAREC
	}
	//If data WAS NOT recieved
	else if(RB0 == 0){
		RB4 = 1;									// set the ACK line
		INTDATAREC = 0;								// clear DATAREC
	}
}//end IntSendData


/********************************
* IntGetData()
*
* This routine gets data from the CD transport
*********************************/
void IntGetData()
{
	char datacount;

	INTRCOMMAND = 0b00000000;							// initallize RCOMMAND to 0
	INTDATAIN = 0b00000000;							// initallize DATAIN to 0

	TRISB = TRISB | 0b00001000;						// set RB3 to input
	TRISB = TRISB & 0b11101111;						// set RB4 to an output
	TRISB = TRISB | 0b00000001;						// set RB0 to an input

	//Sync the PIC with the transport
	while(RB0 == 1){}								// Wait for DATA line to go low
	RB4 = 0;										// Clear the ACK line
	while(RB0 == 0){}								// Wait for DATA line to go high
	RB4 = 1;										// Set the ACK line

	//Recieve Command
	for(datacount = 0; datacount < 8; datacount++)
	{
		while(RB3 == 1){}							// Wait for STB line to be cleared
		INTRCOMMAND = INTRCOMMAND << 1;					// Shift INTRCOMMAND once to the left
		if(RB0 == 1){								// If data bit equals 1
			INTRCOMMAND = INTRCOMMAND | 0b00000001;		// INTRCOMMAND gets INTRCOMMAND ORed with 0x01
		}
		else if(RB0 == 0){							// if data bit equals 0
			INTRCOMMAND = INTRCOMMAND & 0b11111110;		// INTRCOMMAND gets INTRCOMMAND ANDed with 0xfe
		}
		RB4 = 0;									// Clear ACK line
		while(RB3 == 0){}							// Wait for STB line to be set
		RB4 = 1;									// Set ACK line
	}
	//Recieve Data
	for(datacount = 0; datacount < 8; datacount++)
	{
		while(RB3 == 1){}							// Wait for STB line to be cleared
		INTDATAIN = INTDATAIN << 1;						// Shift INTRCOMMAND once to the left
		if(RB0 == 1){								// if data bit equals 1
			INTDATAIN = INTDATAIN | 0b00000001;			// INTDATAIN gets INTDATAIN ORed with 0x01
		}
		else if(RB0 == 0){							// if data bit equals 0
			INTDATAIN = INTDATAIN & 0b11111110;			// INTDATAIN gets INTDATAIN ANDed with 0xfe
		}
		RB4 = 0;									// Clear ACK line
		while(RB3 == 0){}							// Wait for STB line to be set
		RB4 = 1;									// Set ACK line
	}

	TRISB = TRISB | 0b00010000;						// Set RB4 to an input
	TRISB = TRISB & 0b11110111;						// Set RB3 to output
	TRISB = TRISB & 0b11111110;						// set RB0 to output

	//Get acknowledgement from transport
	while(RB4 == 1){}								// Wait for ACK line to be cleared
	RB0 = 1;										// Set DATA line
	RB3 = 0;										// Clear STB line
	while(RB4 == 0){}								// Wait for ACK line to be set
	RB0 = 1;										// Make sure DATA line is set
	RB3 = 1;										// Set STB line
}//end IntGetData



/*************************************INTERRUPT ROUTINES******************************************/

/********************************
* GetTime()
*
* Gets the current track and time from the transport
* during play and displays it on the LCD
*********************************/
void GetTime(){
	char datacount;

	if(PLAYING == 1){									// if playing flag is set
		INTRCOMMAND = 0b00000000;						// initialize RCOMMAND to 0
		INTDATAIN = 0b00000000;							// initialize DATAIN to 0

		TRISB = TRISB | 0b00001000;						// set RB3 to input
		TRISB = TRISB & 0b11101111;						// set RB4 to an output
		TRISB = TRISB | 0b00000001;						// set RB0 to an input

		//Sync the PIC with the transport
		while(RB0 == 1){}								// Wait for DATA line to go low
		RB4 = 0;										// Clear the ACK line
		while(RB0 == 0){}								// Wait for DATA line to go high
		RB4 = 1;										// Set the ACK line

		//Recieve Command
		for(datacount = 0; datacount < 8; datacount++)
		{
			while(RB3 == 1){}							// Wait for STB line to be cleared
			INTRCOMMAND = INTRCOMMAND << 1;					// Shift RCOMMAND once to the left
			if(RB0 == 1){								// If data bit equals 1
				INTRCOMMAND = INTRCOMMAND | 0b00000001;		// RCOMMAND gets RCOMMAND ORed with 0x01
			}
			else if(RB0 == 0){							// if data bit equals 0
				INTRCOMMAND = INTRCOMMAND & 0b11111110;		// RCOMMAND gets RCOMMANd ANDed with 0xfe
			}
			RB4 = 0;									// Clear ACK line
			while(RB3 == 0){}							// Wait for STB line to be set
			RB4 = 1;									// Set ACK line
		}

		//Recieve Data
		for(datacount = 0; datacount < 8; datacount++)
		{
			while(RB3 == 1){}							// Wait for STB line to be cleared
			INTDATAIN = INTDATAIN << 1;						// Shift RCOMMAND once to the left
			if(RB0 == 1){								// if data bit equals 1
				INTDATAIN = INTDATAIN | 0b00000001;			// DATAIN gets DATAIN ORed with 0x01
			}
			else if(RB0 == 0){							// if data bit equals 0
				INTDATAIN = INTDATAIN & 0b11111110;			// DATAIN gets DATAIN ANDed with 0xfe
			}
			RB4 = 0;									// Clear ACK line
			while(RB3 == 0){}							// Wait for STB line to be set
			RB4 = 1;									// Set ACK line
		}

		TRISB = TRISB | 0b00010000;						// Set RB4 to an input
		TRISB = TRISB & 0b11110111;						// Set RB3 to output
		TRISB = TRISB & 0b11111110;						// set RB0 to output

		//Get acknowledgement from transport
		while(RB4 == 1){}								// Wait for ACK line to be cleared
		RB0 = 1;										// Set DATA line
		RB3 = 0;										// Clear STB line
		while(RB4 == 0){}								// Wait for ACK line to be set
		RB0 = 1;										// Make sure DATA line is set
		RB3 = 1;										// Set STB line

		// Display the updated information to the LCD
		if(INTRCOMMAND == 0x13){					// if RCOMMAND = second response
			INTCONV2NUM = INTDATAIN;				// load INTDATAIN into CONV2NUM
			INTPOSITION = 0xd2;						// load LCD INTPOSITION
			IntDisNum();							// display seconds on LCD
		}
		else if(INTRCOMMAND == 0x12){				// if RCOMMAND = minutes response
			INTCONV2NUM = INTDATAIN;				// load INTDATAIN into INTCONV2NUM
			INTPOSITION = 0xcf;						// load LCD INTPOSITION
			IntDisNum();							// display minutes on LCD
		}
		else if(INTRCOMMAND == 0x10){				// if INTRCOMMAND = track number response
			if (INTDATAIN != 0xAA){
				INTCONV2NUM = INTDATAIN;			// load INTDATAIN into INTCONV2NUM
				INTPOSITION = 0xc6;					// load LCD INTPOSITION
				IntDisNum();						// display track number on LCD
			}
			else{
				IntDisplayC(CDENDstr);				// display "END OF CD"
				INTCONV2NUM = CDSEC;
				INTPOSITION = 0xd2;
				IntDisNum();						// display max CD seconds
				INTCONV2NUM = CDMIN;
				INTPOSITION = 0xcf;
				IntDisNum();						// display max CD minutes
				PLAYING = 0;							// clear playing flag
				//insert stop command here

			}
		}
	}
}//end GetTime


/********************************
* PlayIcon()
*
* This routine changes the play icon
*********************************/
void PlayIcon(){
	if(PLAYING == 1 && WAIT > 10){
		PLAYICON[1] = PLAYINUM;
		DisplayV(PLAYICON);
		if (PLAYINUM < 6){
			PLAYINUM = PLAYINUM + 1;
		}
		else {
			PLAYINUM = 1;
		}
		WAIT = 0;
	}
	WAIT = WAIT + 1;
}//end PlayIcon()


/********************************
* IntDisNum()
*
* Takes any number between 0 - 99 converts to ASCII
* and then displays in on the LCD at the position
* specified by INTPOSITION variable
*********************************/
void IntDisNum(){
	char remainder;								// initalize remainder variable

	remainder = INTCONV2NUM % 10;				// mod INTCONV2NUM with 10 store result into remainder
	INTDISNUM[2] = remainder | 0b00110000;		// OR remainder with 0x30 to convert to ASCII and store in string
	INTCONV2NUM = INTCONV2NUM / 10;				// divide INTCONV2NUM by 10
	remainder = INTCONV2NUM % 10;				// mode CONV@NUM and store result into remainder
	INTDISNUM[1] = remainder | 0b00110000;		// OR remainder with 0x03 to convert to ASCII and store in string
	INTDISNUM[0] = INTPOSITION;					// load position code into string
	IntDisplayV(INTDISNUM);						// Display new string
}//end IntDisNum()


/*******************************
 * IntDisplayV(char *)
 *
 * This subroutine is called with the passing in of an array of a variable
 * display string.  It sends the bytes of the string to the LCD.  The first
 * byte sets the cursor position.  The remaining bytes are displayed, beginning
 * at that position.
 *******************************
 */

void IntDisplayV(char * tempPtr)
{
    char currentChar;

    RE0 = 0;                   // Drive RS pin low for cursor-pos. code
    while (*tempPtr)           // if the byte is not zero
    {
        currentChar = *tempPtr;
        RE1 = 1;               // Drive E pin high
        PORTD = currentChar;   // Send upper nibble
        RE1 = 0;               // Drive E pin low so LCD will accept nibble
        currentChar <<= 4;     // Shift lower nibble to upper nibble
        RE1 = 1;               // Drive E pin high again
        PORTD = currentChar;   // Write lower nibble
        RE1 = 0;               // Drive E pin low so LCD will process byte
        IntT40();                 // Wait 40 usec
        RE0 = 1;               // Drive RS pin high for displayable chars
        tempPtr++;             // Increment pointerto next character
    }
}//end IntDisplayV()


/*******************************
 * IntDisplayC(const char *)
 *
 * This subroutine is called with the passing in of an array of a constant
 * display string.  It sends the bytes of the string to the LCD.  The first
 * byte sets the cursor position.  The remaining bytes are displayed, beginning
 * at that position.
 * This subroutine expects a normal one-byte cursor-positioning code, 0xhh, or
 * an occasionally used two-byte cursor-positioning code of the form 0x00hh.
 *******************************
 */

void IntDisplayC(const char * tempPtr)
{
    char currentChar;

    RE0 = 0;                   // Drive RS pin low for cursor-pos code
    while (*tempPtr)           // if the byte is not zero
    {
        currentChar = *tempPtr;
        RE1 = 1;               // Drive E pin high
        PORTD = currentChar;   // Send upper nibble
        RE1 = 0;               // Drive E pin low so LCD will accept nibble
        currentChar <<= 4;     // Shift lower nibble to upper nibble
        RE1 = 1;               // Drive E pin high again
        PORTD = currentChar;   // Write lower nibble
        RE1 = 0;               // Drive E pin low so LCD will process byte
        IntT40();                 // Wait 40 usec
        RE0 = 1;               // Drive RS pin high for displayable chars
        tempPtr++;             // Increment pointerto next character
    }
}//end IntDisplayC()

/*******************************
 * IntT40()
 *
 * Pause for 40 microseconds  or 40/0.4 = 100 clock cycles.
 * Assumes 10/4 = 2.5 MHz internal clock rate.
 *******************************
 */

void IntT40()
{
    unsigned char cCOUNT = 7;
    while (cCOUNT)
    cCOUNT--;
}//end T40

/********************************
* Volume()
*
* Change volume to full
* Temporary function
*********************************/
void Volume(){
	SCOMMAND = 0x51;
	DATAOUT = 0xff;
	SendData();
	GetData();
}//end Volume

/****************************Debugging Routines**********************************/
void DisVar(char VAR){

	char COUNT;
	char TEMP;
	char TEMP2;
	COUNT = 8;
	TEMP2 = VAR;
	while(COUNT)
	{
		TEMP = (TEMP2 & 0b00000001);
		TEMP |= 0x30;
		LONGstr[COUNT] = TEMP;
		TEMP2 = TEMP2>>1;
		COUNT--;
	}
	DisplayV(LONGstr);
}

void IntDisVar(char VAR){

	char COUNT;
	char TEMP;
	char TEMP2;
	COUNT = 8;
	TEMP2 = VAR;
	while(COUNT)
	{
		TEMP = (TEMP2 & 0b00000001);
		TEMP |= 0x30;
		LONGstr[COUNT] = TEMP;
		TEMP2 = TEMP2>>1;
		COUNT--;
	}
	IntDisplayV(LONGstr);
}

void DisVar2(char VAR){

	char COUNT;
	char TEMP;
	char TEMP2;
	COUNT = 8;
	TEMP2 = VAR;
	while(COUNT)
	{
		TEMP = (TEMP2 & 0b00000001);
		TEMP |= 0x30;
		LONGstr2[COUNT] = TEMP;
		TEMP2 = TEMP2>>1;
		COUNT--;
	}
	DisplayV(LONGstr2);
}

/******************************************************************************
 * IMPORTANT: Due to specifics about the compiler, the interrupt service
 * routines *MUST* appear after all other functions in the file.
 ******************************************************************************
 */

/****** HiPriISR interrupt service routine ***********************************/
/*
 * This subroutine handles the high-priority interrupts
 */

void interrupt HiPriISR()
{
	unsigned int CCPR2_TEMP;    // 16-bit timer value

	GIEH = 0;

	if (CCP2IF == 1) {
		TMR3L = 0;              // Clear timers to time next interrupt
	    TMR3H = 0;
	    if (IRLONG == 0)          // Limit new keys to once per 100 ms
	    {
	        REMSTATE = 7;       // Use REMSTATE as a counter to catch the next 6
	                            //  interrupts, ending with REMSTATE = 1
	        RC_CODE = 0;        // Clear RC_CODE
	    }
	    else
	    {
	        if (REMSTATE > 1)                   // REMSTATE > 1 means we should
	        {                                                                       //  use this interrupt
	            REMSTATE--;                     // Decrement REMSTATE counter
	            CCPR2_TEMP = (CCPR2H << 8)      // Convert timer to 16-bit
	                + CCPR2L;
	            if (CCPR2_TEMP < 3750)          // 3750 = 1500 us / .4 us
	            {
	                // Timer value is < 1.5 ms, which indicates a 0
	                RC_CODE = (RC_CODE >> 1)    // Shift RC_CODE right 1 bit
	                    & 0x0f;                 //  and clear upper 4 bits
	            }
	            else if (CCPR2_TEMP < 6000)     // 6000 = 2400 ms / .4 us
	            {
	                // Timer value is > 1.5 ms & < 2.4 ms, indicating a 1
	                RC_CODE = (RC_CODE >> 1)    // Shift RC_CODE right 1 bit
	                    & 0x0f;                 //  and clear upper 4 bits
	                RC_CODE = RC_CODE | 0x10;   // Append a 1 to the RC code
	            }
	        }
	    }
	    IRLONG = 10;              // Reset LONG to 10
	    CCP2IF = 0;             // Clear interrupt flag
	}

	if (INT0IF) {
		GetTime();
		//PlayIcon();
		INT0IF = 0;
	}

	GIEH = 1;
}

/****** LoPriISR interrupt service routine ***********************************/

void interrupt low_priority LoPriISR()  // Low-priority ISR
{

}
